@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: okt 31 2022 (15:05) 
 ## Version: 
-## Last-Updated: mar  8 2023 (09:55) 
+## Last-Updated: jul 26 2023 (11:07) 
 ##           By: Brice Ozenne
-##     Update #: 16
+##     Update #: 38
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -15,7 +15,7 @@
 ## 
 ### Code:
 
-## * manifest
+## * manifest (documentation)
 ##' @title Variables Involved in a Linear Mixed Model
 ##' @description Extract the variables used by the linear mixed model.
 ##'
@@ -26,11 +26,16 @@
 ##' @param original [logical] Should only the variables present in the original data be output?
 ##' When \code{NULL}, variables internally created to fill absent variables will be added to the output.
 ##' When \code{FALSE}, variables internally created are output instead of the original variable for time, cluster, and strata.
+##' @param simplify [logical] Should the list be converted into a vector if a single \code{effects} is requested?
 ##' @param ... not used. For compatibility with the generic function
 ##'
-##' @return A character vector
+##' @return A list of character vectors or a character vector.
+##' 
+##' @keywords methods
+
+## * manifest.lmm 
 ##' @export
-manifest.lmm <- function(x, effects = "all", original = TRUE, ...){
+manifest.lmm <- function(x, effects = "all", original = TRUE, simplify = TRUE, ...){
 
     ## ** extract variables
     if(!is.null(original) && original){
@@ -56,20 +61,42 @@ manifest.lmm <- function(x, effects = "all", original = TRUE, ...){
         }
     }
     valid.effects <- names(ls.out)
+    ## normalize numeric(0) and NA into NULL
+    ls.out[sapply(ls.out, function(iE){sum(!is.na(iE))==0})] <- list(NULL)
 
     ## ** check user input
     dots <- list(...)
     if(length(dots)>0){
         stop("Unknown argument(s) \'",paste(names(dots),collapse="\' \'"),"\'. \n")
     }
-    if(length(effects)==1 && effects == "all"){
-        out <- unlist(ls.out)
+    effects <-  match.arg(effects, c("all",valid.effects), several.ok = TRUE)
+        
+    if(simplify && length(effects)==1){
+        if(effects=="all"){
+            out <- unname(sort(unique(unlist(ls.out))))
+            attributes(out) <- c(attributes(out),ls.out[lengths(ls.out)>0])
+        }else{
+            out <- unname(sort(unique(unlist(ls.out[effects]))))
+        }
+        
     }else{
-        effects <-  match.arg(effects, valid.effects, several.ok = TRUE)
-        out <- unlist(ls.out[effects])
+        out <- ls.out[effects]
     }
-    return(out[!is.na(out)])
+    return(out)
 }
 
+## * manifest.mlmm 
+##' @export
+manifest.mlmm <- function(x, ...){
+    ls.manifest <- lapply(x$model, manifest)
+
+    if(any(sapply(ls.manifest,identical,ls.manifest[[1]])==FALSE)){
+        stop("Difference in manifest variables between the LMM. \n",
+             "Cannot provide a single output for all models.")
+    }
+    out <- union(ls.manifest[[1]], x$object$by)
+    attributes(out) <- c(attributes(ls.manifest[[1]]), list(by = x$object$by))
+    return(out)
+}
 ##----------------------------------------------------------------------
 ### manifest.R ends here
