@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: okt 20 2021 (11:00) 
 ## Version: 
-## Last-Updated: jan 23 2023 (16:04) 
+## Last-Updated: jul 27 2023 (13:50) 
 ##           By: Brice Ozenne
-##     Update #: 109
+##     Update #: 119
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -15,22 +15,21 @@
 ## 
 ### Code:
 
-## * plot (code)
+## * plot.lmm (code)
 ##' @describeIn autoplot.lmm Graphical Display For Linear Mixed Models
 ##' @export
 plot.lmm <- function(x, ...){
-
     out <- autoplot.lmm(x, ...)
     if(!is.null(out$plot)){
         ## if ggplot then the graph is stored in out and should be displayed here
-        ## otherwise the graph has already been displayed but could not be stored in the object
+        ## otherwise the graph has already been displayed but could not be stored in the object (e.g. when using qqtest)
         print(out$plot)
     }
     return(invisible(out))
 
 }
 
-## * plot (code)
+## * plot.partialCor (code)
 ##' @describeIn autoplot.partialCor Graphical Display For Partial Correlation
 ##' @export
 plot.partialCor <- function(x, ...){
@@ -40,7 +39,7 @@ plot.partialCor <- function(x, ...){
 
 }
 
-## * plot (code)
+## * plot.profil_lmm (code)
 ##' @describeIn autoplot.profile_lmm Display Contour of the log-Likelihood
 ##' @export
 plot.profile_lmm <- function(x, ...){
@@ -50,7 +49,7 @@ plot.profile_lmm <- function(x, ...){
 
 }
 
-## * plot (code)
+## * plot.residuals_lmm (code)
 ##' @describeIn autoplot.residuals_lmm Graphical Display of the Residuals
 ##' @export
 plot.residuals_lmm <- function(x, ...){
@@ -60,7 +59,16 @@ plot.residuals_lmm <- function(x, ...){
 
 }
 
-## * plot (code)
+## * plot.summarize (code)
+##' @describeIn autoplot.summarize Graphical Display of Missing Data Pattern
+##' @export
+plot.summarize <- function(x, ...){
+    out <- autoplot.summarize(x, ...)
+    print(out$plot)
+    return(invisible(out))
+}
+
+## * plot.summarizeNA (code)
 ##' @describeIn autoplot.summarizeNA Graphical Display of Missing Data Pattern
 ##' @export
 plot.summarizeNA <- function(x, ...){
@@ -69,7 +77,7 @@ plot.summarizeNA <- function(x, ...){
     return(invisible(out))
 }
 
-## * plot (code)
+## * plot.Wald_lmm (code)
 ##' @describeIn autoplot.Wald_lmm Graphical Display For Linear Hypothesis Test
 ##' @export
 plot.Wald_lmm <- function(x, ...){
