@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: sep 16 2021 (13:18) 
 ## Version: 
-## Last-Updated: feb 27 2023 (17:43) 
+## Last-Updated: jul 26 2023 (14:26) 
 ##           By: Brice Ozenne
-##     Update #: 176
+##     Update #: 211
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -76,18 +76,17 @@
     name.k <- object$param$name[type=="k"]
     name.rho <- object$param$name[type=="rho"]
     name.paramVar <- c(name.sigma,name.k,name.rho)
-
     param <- param[name.paramVar]
     name.param <- names(param)
     if(missing(Omega)){
         Omega <- .calc_Omega(object, param = param, keep.interim = TRUE)
     }
     
-    Upattern <- object$X$Upattern
+    Upattern <- object$Upattern
     n.Upattern <- NROW(Upattern)
-    pattern.cluster <- object$X$pattern.cluster$pattern
-    X.var <- object$X$Xpattern.var
-    X.cor <- object$X$Xpattern.cor
+    pattern.cluster <- object$pattern
+    X.var <- object$var$Xpattern
+    X.cor <- object$cor$Xpattern
 
     if(identical(transform.sigma,"log") && identical(transform.k,"log") && identical(transform.rho,"atanh")){
         Jacobian <- NULL
@@ -164,6 +163,7 @@
         }
         return(iScore)
     })
+
     ## ** export
     out <- stats::setNames(out,Upattern$name)
     return(out)
@@ -175,6 +175,9 @@
 ## * calc_dOmega.CS
 .calc_dOmega.CS <- .calc_dOmega.ID
 
+## * calc_dOmega.RE
+.calc_dOmega.RE <- .calc_dOmega.ID
+
 ## * calc_dOmega.TOEPLITZ
 .calc_dOmega.TOEPLITZ <- .calc_dOmega.ID
 
@@ -185,15 +188,16 @@
 .calc_dOmega.CUSTOM <- function(object, param, Omega, Jacobian = NULL,
                                 transform.sigma = NULL, transform.k = NULL, transform.rho = NULL){
 
-    Upattern <- object$X$Upattern
+    Upattern <- object$Upattern
     n.Upattern <- NROW(Upattern)
-
+    X.var <- object$var$Xpattern
+    X.cor <- object$cor$Xpattern
     FCT.sigma <- object$FCT.sigma
     FCT.rho <- object$FCT.rho
     dFCT.sigma <- object$dFCT.sigma
     dFCT.rho <- object$dFCT.rho
-    name.sigma <- names(object$init.sigma)
-    name.rho <- names(object$init.rho)
+    name.sigma <- object$param[object$param$type=="sigma","name"]
+    name.rho <- object$param[object$param$type=="rho","name"]
 
     if(!is.null(FCT.sigma) && is.null(dFCT.sigma) || !is.null(FCT.rho) && is.null(dFCT.rho) ){
 
@@ -203,10 +207,10 @@
         }, x = param[c(name.sigma,name.rho)])
 
         vec.pattern <- unlist(lapply(names(Omega), function(iName){
-            iTime <- object$X$Upattern$time[[iName]]            
             iNtime <- Upattern[Upattern$name==iName,"n.time"]
-            iOut <- matrix(iName, nrow = iNtime, ncol = iNtime, dimnames = list(iTime,iTime))
+            iOut <- matrix(iName, nrow = iNtime, ncol = iNtime)
         }))
+        
         out <- by(data = vec.dOmega, INDICES = vec.pattern, FUN = function(idOmega){ ## idOmega <- vec.dOmega[1:16,]
             iOut <- apply(idOmega, MARGIN = 2, simplify = FALSE, function(iVec){
                 iNtime <- sqrt(length(iVec))
@@ -214,35 +218,31 @@
             })
             names(iOut) <- c(name.sigma,name.rho)
             return(iOut)
-        })
+        }, simplify = FALSE)
         class(out) <- "list"
         attr(out,"call") <- NULL
 
     }else{
 
-        pattern.cluster <- object$X$pattern.cluster
-        X.var <- object$X$var
-        X.cor <- object$X$cor
-
         out <- stats::setNames(lapply(1:n.Upattern, function(iPattern){ ## iPattern <- 1
+
             ## derivative of sd with respect to the variance parameters
-            iPattern.var <- object$X$Upattern$var[iPattern]
-            iNtime <- object$X$Upattern$n.time[iPattern]
-            iX.var <- object$X$Xpattern.var[[iPattern.var]]
-            iTime <- object$X$Upattern$time[[iPattern]]
+            iPattern.var <- Upattern$var[iPattern]
+            iNtime <- Upattern$n.time[iPattern]
+            iX.var <- X.var[[iPattern.var]]
             iOmega.sd <- attr(Omega[[iPattern]], "sd")
-            idOmega.sd <- dFCT.sigma(p = param[name.sigma], time = iTime, X = iX.var)
+            idOmega.sd <- dFCT.sigma(p = param[name.sigma], n.time = iNtime, X = iX.var)
 
             ## derivative of rho with respect to the correlation parameters
-            if(iNtime > 1 && !is.null(X.cor)){
-                iPattern.cor <- object$X$Upattern$cor[iPattern]
-                iX.cor <- object$X$Xpattern.cor[[iPattern.cor]]
+            if(iNtime > 1 && !is.na(Upattern$cor[iPattern])){
+                iPattern.cor <- Upattern$cor[iPattern]
+                iX.cor <- X.cor[[iPattern.cor]]
                 iOmega.cor <- attr(Omega[[iPattern]], "cor")
-                idOmega.cor <- dFCT.rho(p = param[name.rho], time = iTime, X = iX.cor)
+                idOmega.cor <- dFCT.rho(p = param[name.rho], n.time = iNtime, X = iX.cor)
             }
 
             ## derivative of Omega with respect to the variance and correlation parameters
-            if(iNtime > 1 && !is.null(X.cor)){
+            if(iNtime > 1 && !is.na(Upattern$cor[iPattern])){
                 iOut <- c(
                     lapply(idOmega.sd, function(iDeriv){
                         iDeriv <- unname(iDeriv)
